#!/usr/bin/env node
"use strict";
/* eslint-disable @typescript-eslint/no-explicit-any */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fast_glob_1 = (0, tslib_1.__importDefault)(require("fast-glob"));
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var fs_1 = (0, tslib_1.__importDefault)(require("fs"));
var yargs_parser_1 = (0, tslib_1.__importDefault)(require("yargs-parser"));
var semver_1 = (0, tslib_1.__importDefault)(require("semver"));
var jest_1 = require("jest");
var helpers_1 = require("yargs/helpers");
var test_at_scale_core_1 = require("@lambdatest/test-at-scale-core");
var constants_1 = require("./constants");
var JestRunner = /** @class */ (function () {
    function JestRunner() {
        this.setEnv();
    }
    JestRunner.prototype.setEnv = function () {
        // Jest CLI will set process.env.NODE_ENV to 'test' when it's null, do the same here
        // https://github.com/facebook/jest/blob/master/packages/jest-cli/bin/jest.js#L12-L14
        if (!process.env.NODE_ENV) {
            process.env.NODE_ENV = 'test';
        }
    };
    JestRunner.prototype.discoverTests = function (argv) {
        var _a, _b, _c;
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var repoID, buildID, taskID, orgID, commitID, branch, testFilesGlob, testFiles, changedFilesSet, result, tests, testSuites, _d, impactedTests, executeAllTests, discoveryResult;
            return (0, tslib_1.__generator)(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        test_at_scale_core_1.Validations.validateDiscoveryEnv(argv);
                        repoID = process.env.REPO_ID;
                        buildID = process.env.BUILD_ID;
                        taskID = process.env.TASK_ID;
                        orgID = process.env.ORG_ID;
                        commitID = process.env.COMMIT_ID;
                        branch = process.env.BRANCH_NAME;
                        testFilesGlob = argv.pattern;
                        testFiles = fast_glob_1.default.sync(testFilesGlob).map(function (file) { return path_1.default.resolve(file); });
                        if (testFiles.length === 0) {
                            return [2 /*return*/, new test_at_scale_core_1.DiscoveryResult([], [], [], repoID, commitID, buildID, taskID, orgID, branch)];
                        }
                        changedFilesSet = argv.diff === undefined ? undefined : new Set(argv.diff);
                        return [4 /*yield*/, this.runJest(testFiles, constants_1.MATCH_NOTHING_REGEX, [require.resolve("./jest-discover-reporter")])];
                    case 1:
                        _e.sent();
                        return [4 /*yield*/, test_at_scale_core_1.JSONStream.parse(fs_1.default.createReadStream(constants_1.DISCOVERY_RESULT_FILE))];
                    case 2:
                        result = (_a = (_e.sent())) !== null && _a !== void 0 ? _a : {};
                        tests = ((_b = result.tests) !== null && _b !== void 0 ? _b : []).map(function (test) { return test_at_scale_core_1.Test.fromJSON(test); });
                        testSuites = ((_c = result.testSuites) !== null && _c !== void 0 ? _c : []).map(test_at_scale_core_1.TestSuite.fromJSON);
                        test_at_scale_core_1.Util.handleDuplicateTests(tests);
                        return [4 /*yield*/, test_at_scale_core_1.Util.findImpactedTests(tests, changedFilesSet, testFiles)];
                    case 3:
                        _d = tslib_1.__read.apply(void 0, [_e.sent(), 2]), impactedTests = _d[0], executeAllTests = _d[1];
                        discoveryResult = new test_at_scale_core_1.DiscoveryResult(tests, testSuites, impactedTests, repoID, commitID, buildID, taskID, orgID, branch, executeAllTests);
                        test_at_scale_core_1.Util.fillTotalTests(discoveryResult);
                        return [2 /*return*/, discoveryResult];
                }
            });
        });
    };
    JestRunner.prototype.execute = function (testFilesGlob, locators) {
        if (locators === void 0) { locators = []; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var testLocators, testFilesToProcess, testFiles, testFilesToProcessList, _a, regex, blockTestLocators, executionResult, _b, _c;
            return (0, tslib_1.__generator)(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        testLocators = new Set(locators);
                        testFilesToProcess = new Set();
                        if (testLocators.size == 0) {
                            testFiles = fast_glob_1.default.sync(testFilesGlob).map(function (file) { return path_1.default.resolve(file); });
                            testFilesToProcess = new Set(testFiles);
                        }
                        else {
                            testFilesToProcess = test_at_scale_core_1.Util.getFilesFromTestLocators(testLocators);
                        }
                        testFilesToProcessList = Array.from(testFilesToProcess);
                        if (testFilesToProcessList.length == 0) {
                            return [2 /*return*/, new test_at_scale_core_1.ExecutionResult()];
                        }
                        _a = (0, tslib_1.__read)(this.getBlockTestAndTestRegex(testFilesToProcessList, testLocators), 2), regex = _a[0], blockTestLocators = _a[1];
                        return [4 /*yield*/, this.runJest(testFilesToProcessList, regex, [require.resolve("./jest-reporter")], true)];
                    case 1:
                        _d.sent();
                        _c = (_b = test_at_scale_core_1.ExecutionResult).fromJSON;
                        return [4 /*yield*/, test_at_scale_core_1.JSONStream.parse(fs_1.default.createReadStream(constants_1.REPORT_FILE))];
                    case 2:
                        executionResult = _c.apply(_b, [_d.sent()]);
                        test_at_scale_core_1.Util.handleDuplicateTests(executionResult.testResults);
                        if (locators.length > 0) {
                            executionResult.testResults = test_at_scale_core_1.Util.filterTestResultsByTestLocator(executionResult.testResults, testLocators, blockTestLocators);
                            if (executionResult.testSuiteResults.length > 0) {
                                executionResult.testSuiteResults = test_at_scale_core_1.Util.filterTestSuiteResults(executionResult.testResults, executionResult.testSuiteResults);
                            }
                        }
                        return [2 /*return*/, executionResult];
                }
            });
        });
    };
    JestRunner.prototype.executeTests = function (argv) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var taskID, buildID, orgID, repoID, commitID, testFilesGlob, locatorFile, executionResults, locators, result, result;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        taskID = process.env.TASK_ID;
                        buildID = process.env.BUILD_ID;
                        orgID = process.env.ORG_ID;
                        repoID = process.env.REPO_ID;
                        commitID = process.env.COMMIT_ID;
                        test_at_scale_core_1.Validations.validateExecutionEnv(argv);
                        testFilesGlob = argv.pattern;
                        locatorFile = argv.locatorFile;
                        executionResults = new test_at_scale_core_1.ExecutionResults(taskID, buildID, repoID, commitID, orgID);
                        if (!locatorFile) return [3 /*break*/, 2];
                        locators = test_at_scale_core_1.Util.getLocatorsFromFile(locatorFile);
                        return [4 /*yield*/, this.execute(testFilesGlob, locators)];
                    case 1:
                        result = _a.sent();
                        executionResults.push(result);
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, this.execute(testFilesGlob)];
                    case 3:
                        result = _a.sent();
                        executionResults.push(result);
                        _a.label = 4;
                    case 4: return [2 /*return*/, executionResults];
                }
            });
        });
    };
    JestRunner.prototype.runJest = function (testFilesToProcessList, testNamePattern, reporters, inExecutionPhase) {
        if (inExecutionPhase === void 0) { inExecutionPhase = false; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var argv, jestArgv, jestCLIModuleDir, readConfig, _a, globalConfig, projectConfig;
            return (0, tslib_1.__generator)(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        argv = (0, yargs_parser_1.default)((0, helpers_1.hideBin)(process.argv));
                        jestArgv = {
                            $0: "jest-runner",
                            _: testFilesToProcessList,
                            testNamePattern: testNamePattern,
                            config: argv.config,
                            collectCoverage: inExecutionPhase && !!process.env.TAS_COLLECT_COVERAGE,
                            cacheDirectory: constants_1.JEST_CACHE_DIR
                        };
                        if (!inExecutionPhase) return [3 /*break*/, 3];
                        jestCLIModuleDir = path_1.default.dirname(require.resolve("jest-cli/package.json"));
                        module.paths.push(path_1.default.join(jestCLIModuleDir, "node_modules"));
                        return [4 /*yield*/, Promise.resolve().then(function () { return (0, tslib_1.__importStar)(require("jest-config")); })];
                    case 1:
                        readConfig = (_b.sent()).readConfig;
                        return [4 /*yield*/, readConfig(jestArgv, test_at_scale_core_1.Util.REPO_ROOT)];
                    case 2:
                        _a = _b.sent(), globalConfig = _a.globalConfig, projectConfig = _a.projectConfig;
                        if (semver_1.default.lt((0, jest_1.getVersion)(), "24.0.0")) {
                            jestArgv.setupTestFrameworkScriptFile = constants_1.SETUP_AFTER_ENV_FILE;
                        }
                        else {
                            jestArgv.setupFilesAfterEnv = [constants_1.SETUP_AFTER_ENV_FILE].concat(projectConfig.setupFilesAfterEnv);
                        }
                        if (!globalConfig.reporters) {
                            reporters = reporters.concat(["default"]);
                        }
                        else {
                            reporters = reporters.concat(globalConfig.reporters);
                        }
                        jestArgv.runInBand = true;
                        return [3 /*break*/, 4];
                    case 3:
                        jestArgv.silent = true;
                        _b.label = 4;
                    case 4:
                        jestArgv.reporters = reporters;
                        return [4 /*yield*/, (0, jest_1.runCLI)(jestArgv, [test_at_scale_core_1.Util.REPO_ROOT])];
                    case 5:
                        _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    JestRunner.prototype.getBlockTestAndTestRegex = function (testFiles, testLocators) {
        var e_1, _a, e_2, _b, e_3, _c;
        var filteredTestsRegexes = [];
        var blockTestLocators = new Set();
        if (testLocators.size > 0) {
            try {
                for (var testLocators_1 = (0, tslib_1.__values)(testLocators), testLocators_1_1 = testLocators_1.next(); !testLocators_1_1.done; testLocators_1_1 = testLocators_1.next()) {
                    var locator = testLocators_1_1.value;
                    var loc = test_at_scale_core_1.Locator.from(locator);
                    if (!loc) {
                        continue;
                    }
                    var blockTest = test_at_scale_core_1.Util.getBlockTestLocatorProperties(loc);
                    var parts = loc.child ? loc.child.toString().split(test_at_scale_core_1.LocatorSeparator) : [];
                    // parts = [file.js, testSuite1, testSuite2, testName]
                    parts = parts.filter(function (part) { return part.length > 0; });
                    if (parts.length > 0) {
                        var testFullName = parts.join(" ");
                        // testRegex = (^testSuite1 testSuite2 testName$)
                        var testRegex = "(^" + test_at_scale_core_1.Util.escapeRegExp(testFullName) + "$)";
                        if (!blockTest.isBlocked) {
                            filteredTestsRegexes.push(testRegex);
                        }
                        else {
                            blockTestLocators.add(loc.toString());
                        }
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (testLocators_1_1 && !testLocators_1_1.done && (_a = testLocators_1.return)) _a.call(testLocators_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            // if all tests in the locators were blocked then we will execute nothing
            if (filteredTestsRegexes.length == 0) {
                return [constants_1.MATCH_NOTHING_REGEX, blockTestLocators];
            }
            // (^testSuite1 testSuite2 testName$)|(^testSuite3 testSuite4 testName2$)
            return [filteredTestsRegexes.join("|"), blockTestLocators];
        }
        // in case no test locators specified we will execute all tests 
        // but first filter out blocktest ones
        var blockTestsRegexes = [];
        try {
            for (var testFiles_1 = (0, tslib_1.__values)(testFiles), testFiles_1_1 = testFiles_1.next(); !testFiles_1_1.done; testFiles_1_1 = testFiles_1.next()) {
                var testFile = testFiles_1_1.value;
                var relFilePath = path_1.default.relative(test_at_scale_core_1.Util.REPO_ROOT, testFile);
                var blockTestLocators_2 = test_at_scale_core_1.Util.getBlockTestLocatorsForFile(relFilePath).map(function (item) { return item.locator; });
                try {
                    for (var blockTestLocators_1 = (e_3 = void 0, (0, tslib_1.__values)(blockTestLocators_2)), blockTestLocators_1_1 = blockTestLocators_1.next(); !blockTestLocators_1_1.done; blockTestLocators_1_1 = blockTestLocators_1.next()) {
                        var blockTestLocator = blockTestLocators_1_1.value;
                        var parts = blockTestLocator.split(test_at_scale_core_1.LocatorSeparator);
                        // parts = [file.js, testSuite1, testSuite2, testName]
                        parts = parts.filter(function (part) { return part.length > 0; });
                        parts.shift();
                        if (parts.length > 0) {
                            var testFullName = parts.join(" ");
                            // testRegex = (^testSuite1 testSuite2 testName)
                            var testRegex = "(^" + test_at_scale_core_1.Util.escapeRegExp(testFullName) + ")";
                            blockTestsRegexes.push(testRegex);
                        }
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (blockTestLocators_1_1 && !blockTestLocators_1_1.done && (_c = blockTestLocators_1.return)) _c.call(blockTestLocators_1);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (testFiles_1_1 && !testFiles_1_1.done && (_b = testFiles_1.return)) _b.call(testFiles_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        // if only blocked tests exist, then return block tests regex
        if (blockTestsRegexes.length > 0) {
            // ^(?!(^testSuite1 testSuite2 testName)|(^testSuite3 testSuite4 testName2)).*$
            return ["^(?!" + blockTestsRegexes.join("|") + ").*$", blockTestLocators];
        }
        return ["", blockTestLocators];
    };
    return JestRunner;
}());
(function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var runner, argv, result, endpoint, err_1, e_4;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                runner = new JestRunner();
                _a.label = 1;
            case 1:
                _a.trys.push([1, 11, , 12]);
                argv = (0, yargs_parser_1.default)((0, helpers_1.hideBin)(process.argv), {
                    array: ['diff', "locator"],
                    configuration: { 'strip-dashed': true },
                });
                if (!argv.command) {
                    throw Error("Command not provided.");
                }
                result = void 0;
                endpoint = void 0;
                if (!(argv.command === "discover")) return [3 /*break*/, 3];
                return [4 /*yield*/, runner.discoverTests(argv)];
            case 2:
                result = _a.sent();
                endpoint = process.env.ENDPOINT_POST_TEST_LIST || "";
                return [3 /*break*/, 6];
            case 3:
                if (!(argv.command === "execute")) return [3 /*break*/, 5];
                return [4 /*yield*/, runner.executeTests(argv)];
            case 4:
                result = _a.sent();
                endpoint = process.env.ENDPOINT_POST_TEST_RESULTS || "";
                return [3 /*break*/, 6];
            case 5: throw Error("Unknown/Not implemented command");
            case 6:
                if (!endpoint) return [3 /*break*/, 10];
                _a.label = 7;
            case 7:
                _a.trys.push([7, 9, , 10]);
                return [4 /*yield*/, test_at_scale_core_1.Util.makeApiRequestPost(endpoint, result)];
            case 8:
                _a.sent();
                return [3 /*break*/, 10];
            case 9:
                err_1 = _a.sent();
                throw new test_at_scale_core_1.RunnerException(err_1.stack);
            case 10: return [3 /*break*/, 12];
            case 11:
                e_4 = _a.sent();
                console.error(e_4.stack);
                process.exit(-1);
                return [3 /*break*/, 12];
            case 12:
                process.exit(0);
                return [2 /*return*/];
        }
    });
}); })();
//# sourceMappingURL=jest-runner.js.map